%  Mie-Scattering Experiment for the Classroom Manufactured by 3D Printing
%  Christian Scholz, Achim Sack, Michael Heckel, and Thorsten Pöschel
%  2016
%  
%  http://www.mss.cbi.fau.de/Mie-experiment/
%  Institute for Multiscale Simluation
%  Naegelsbachstr. 49b
%  91054 Erlangen
%  Germany
%
%  Determine minimal distance between two vectors A and B, where length(B)<length(A)

function MinDist = findMinDist(A,B)
%  Find the minimum distance between two vectors containing the angular position 
%  of the extrema in ascending order
%
if length(B)<length(A)
  % determine size difference between A and B
  dl=length(A)-length(B);

  % repeat B "length(A)-length(B),1)"-times
  % Fill B with NaNs to equal length
  B=[B; ones(dl,1)/NaN];
  
  % create Matrix to store circshifted Bs
  SB=B;

  for i=1:dl
    SB=[SB,circshift(B,i)];
  end

  % compute collumn wise minus and set NaNs to 0
  Vdistance = bsxfun(@minus,SB,A);
  Vdistance(~isfinite(Vdistance))=0;

  % calculate distance for each collumn and find minimum
  N = sqrt(sum(abs(Vdistance).^2,1));
  MinDist=min(N);
else
  % if B is larger than A the vectors cannot match, so return NaN
  MinDist=NaN;
end
