%  Mie-Scattering Experiment for the Classroom Manufactured by 3D Printing
%  Christian Scholz, Achim Sack, Michael Heckel, and Thorsten Pöschel
%  2016
%
%  http://www.mss.cbi.fau.de/Mie-experiment/
%  Institute for Multiscale Simluation
%  Naegelsbachstr. 49b
%  91054 Erlangen
%  Germany
%
%  Computing table of extrema for angular Mie-scattering intensity
%  Computation of angular Mie-scattering intensity based on
%  s. p. 111-114, Bohren and Huffman (1983) BEWI:TDD122
%  and the Matlab implementation by
%  C. Mätzler, May 2002
%  
%  Get position of measured extrema using smooth.m on the measured data

% measured extrema for 3 µm, determine with smooth.m
%Measure=[10.90 16.64 24.29 29.42 38.87]';
%Measure=[10.5 16.2 24.5 29.0 40.4]';
% measured extrema for 5.9 µm, determine with smooth.m
%Measure=[11.0 15.3 20.3 22.7 27.2 30.5 34.7 38.0 42.8 45.9]';
% measured extrema for 4.52 µm, determine with smooth.m
Measure=[23.7 26.6 35.0 36.6 43.7 46.0]';

% set material parameters
npar=1.58;
nmed=1.33;
lambda=0.635;

% set minimum and maximum angle and number of steps
amin=0;
amax=48.75/180*pi; % here: define maximum angle from the critical angle of refrection
nsteps=360;

% set range of particle diameters in µm 
dmin= 1.0;
dmax= 8.0;
dd= 0.01;

% init container and calculate mie table
mtable=cell(length(dmin:dd:dmax));

% range of diameters
diams=dmin:dd:dmax;

i=1; % increment used for counting
for diam=diams;
  res=mie_tetascan_lim(npar/nmed,pi*diam/(lambda/nmed),amin,amax,nsteps);
  [ymax,imax,ymin,imin] = extrema(res(:,2));	%for perpendicular intensities select res(:,2) and for parallel res(:,3)
  E=sort([res(imax,1)*180/pi; res(imin,1)*180/pi]);
  %discard first and last extrema
  E(1)=[];
  E(end)=[];
  % correct e for snells law
  E=asind(nmed*sind(E));
  mtable{i}=E;
  i=i+1;
end

% search for minimum
dmin=length(mtable);
for i=1:1:length(mtable)
  dmin(i)=findMinDist(mtable{i},Measure);
end

% determine and print posiiton of global minimum 
[a,b]=min(dmin);
disp('Position of global minimum is:');
diams(b)
