%  Mie-Scattering Experiment for the Classroom Manufactured by 3D Printing
%  Christian Scholz, Achim Sack, Michael Heckel, and Thorsten Pöschel
%  2016
%  
%  http://www.mss.cbi.fau.de/Mie-experiment/
%  Institute for Multiscale Simluation
%  Naegelsbachstr. 49b
%  91054 Erlangen
%  Germany
%
%  Reads experimental data from inFile, determines extrema in a specified range 
%  from a spline fit and returns data position and value to outFile.

% set input and output
inFile='Mie_scattering_parallel_45u_160316_2.dat';
outFile='Mie_scattering_parallel_extrema_45u_160316_2.dat';

% load experimental data
A=dlmread(inFile);

% set the smoothing distance
sd=3;

% adjust minimum and maximum angle
amin=-50;
amax=50;

% interpolate experimental data via spline-fit
angles=amin:0.01:amax;
ys = interp1 (A(1:sd:end,1),A(1:sd:end,2),angles,"spline");

% determine extrema of spline
[ymax,imax,ymin,imin] = extrema(ys);

% sort extrema by ascending order
E=sort([angles(1,imax) angles(1,imin)]);
E(end)=[];
E(1)=[];

% write extrema to ouput
dlmwrite(outFile,E);
